@@ -33,7 +33,7 @@ from genshi.template.eval import _ast
 from genshi.template.base import DirectiveFactory, EXPR, SUB, _apply_directives
 from genshi.template.directives import Directive, StripDirective
 from genshi.template.markup import MarkupTemplate, EXEC
-from genshi.compat import IS_PYTHON2
+from genshi.compat import IS_PYTHON2, _ast_Str, _ast_Str_value
 
 __all__ = ['Translator', 'extract']
 __docformat__ = 'restructuredtext en'
@@ -604,7 +604,8 @@ class Translator(DirectiveFactory):
         QName('style'), QName('http://www.w3.org/1999/xhtml}style')
     ])
     INCLUDE_ATTRS = frozenset([
-        'abbr', 'alt', 'label', 'prompt', 'standby', 'summary', 'title'
+        'abbr', 'alt', 'label', 'prompt', 'standby', 'summary', 'title',
+        'placeholder',
     ])
     NAMESPACE = I18N_NAMESPACE
 
@@ -981,7 +982,7 @@ class MessageBuffer(object):
         elif kind is TEXT:
             if '[' in data or ']' in data:
                 # Quote [ and ] if it ain't us adding it, ie, if the user is
-                # using those chars in his templates, escape them
+                # using those chars in their templates, escape them
                 data = data.replace('[', '\[').replace(']', '\]')
             self.string.append(data)
             self._add_event(self.stack[-1], (kind, data, pos))
@@ -1186,10 +1187,11 @@ def extract_from_code(code, gettext_functions):
                 and node.func.id in gettext_functions:
             strings = []
             def _add(arg):
-                if isinstance(arg, _ast.Str) and isinstance(arg.s, str):
-                    strings.append(arg.s)
-                elif isinstance(arg, _ast.Str):
-                    strings.append(str(arg.s, 'utf-8'))
+                if isinstance(arg, _ast_Str) \
+                        and isinstance(_ast_Str_value(arg), str):
+                    strings.append(_ast_Str_value(arg))
+                elif isinstance(arg, _ast_Str):
+                    strings.append(str(_ast_Str_value(arg), 'utf-8'))
                 elif arg:
                     strings.append(None)
             [_add(arg) for arg in node.args]
